@@ -53,7 +53,7 @@
 
 #define THUMBNAIL_DIR_PERMISSIONS 0700
 
-typedef struct
+struct _ThumbLoaderPrivateData
 {
 	ImageLoader *il;
 
@@ -63,7 +63,7 @@ typedef struct
 				    * image when done */
 
 	char *uri;
-	char *path;
+	char *e_uri;
 
 	gboolean use_cache : 1;
 	gboolean from_cache : 1;
@@ -80,7 +80,7 @@ typedef struct
 					    * than this the thumbnail will 
 					    * not be created, for 
 					    * functionality reasons. */
-} ThumbLoaderPrivateData;
+};
 
 
 enum {
@@ -129,7 +129,7 @@ thumb_loader_finalize (GObject *object)
 	g_object_unref (G_OBJECT (priv->il));
 
 	g_free (priv->uri);
-	g_free (priv->path);
+	g_free (priv->e_uri);
 
 	g_free (priv);
 	tl->priv = NULL;
@@ -192,7 +192,7 @@ thumb_loader_init (ThumbLoader *tl)
 
 	priv->thumb_factory = NULL;
 	priv->uri = NULL;
-	priv->path = NULL;
+	priv->e_uri = NULL;
 	priv->pixbuf = NULL;
 	priv->use_cache = TRUE;
 	priv->from_cache = FALSE;
@@ -234,11 +234,11 @@ thumb_loader (const char  *path,
 	      GError     **error,
 	      gpointer     data)
 {
-	ThumbLoader        *tl = data;
 	GdkPixbufAnimation *animation = NULL;
 
 #ifdef HAVE_LIBJPEG
 	if (image_is_jpeg (path)) {
+		ThumbLoader *tl = data;
 		ThumbLoaderPrivateData *priv = tl->priv;
 		GdkPixbuf *pixbuf;
 
@@ -305,7 +305,7 @@ thumb_loader_new (const char *path,
 		thumb_loader_set_path (tl, path);
 	else {
 		priv->uri = NULL;
-		priv->path = NULL;
+		priv->e_uri = NULL;
 	}
 
 	priv->il = IMAGE_LOADER (image_loader_new (path, FALSE));
@@ -368,30 +368,16 @@ void
 thumb_loader_set_path (ThumbLoader *tl,
 		       const char  *path)
 {
-	ThumbLoaderPrivateData *priv;
-	char        *escaped_path;
-	GnomeVFSURI *vfs_uri;
-
 	g_return_if_fail (tl != NULL);
 	g_return_if_fail (path != NULL);
 
-	priv = tl->priv;
-	g_free (priv->uri);
-	g_free (priv->path);
+	g_free (tl->priv->uri);
+	g_free (tl->priv->e_uri);
 
-	vfs_uri = new_uri_from_path (path);
+	tl->priv->uri = get_uri_from_path (path);
+	tl->priv->e_uri = gnome_vfs_escape_host_and_path_string (tl->priv->uri);
 
-	escaped_path = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
-	priv->uri = gnome_vfs_unescape_string (escaped_path, NULL);
-	g_free (escaped_path);
-
-	escaped_path = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
-	priv->path = gnome_vfs_unescape_string (escaped_path, NULL);
-	g_free (escaped_path);
-
-	gnome_vfs_uri_unref (vfs_uri);
-
-	image_loader_set_path (priv->il, priv->path);
+	image_loader_set_path (tl->priv->il, remove_scheme_from_uri (tl->priv->uri));
 }
 
 
@@ -406,10 +392,11 @@ thumb_loader_set_uri (ThumbLoader       *tl,
 
 	priv = tl->priv;
 	g_free (priv->uri);
-	g_free (priv->path);
+	g_free (priv->e_uri);
 
-	priv->uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
-	priv->path = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+	
+	priv->e_uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
+	priv->uri = gnome_vfs_unescape_string (priv->e_uri, NULL);
 
 	image_loader_set_uri (priv->il, vfs_uri);
 }
@@ -418,49 +405,36 @@ thumb_loader_set_uri (ThumbLoader       *tl,
 GnomeVFSURI *
 thumb_loader_get_uri (ThumbLoader *tl)
 {
-	ThumbLoaderPrivateData *priv;
-
 	g_return_val_if_fail (tl != NULL, NULL);
-
-	priv = tl->priv;
-
-	return new_uri_from_path (priv->path);
+	return new_uri_from_path (tl->priv->uri);
 }
 
 
 GdkPixbuf *
 thumb_loader_get_pixbuf (ThumbLoader *tl)
 {
-	ThumbLoaderPrivateData *priv;
-
 	g_return_val_if_fail (tl != NULL, NULL);
-
-	priv = tl->priv;
-	return priv->pixbuf;
+	return tl->priv->pixbuf;
 }
 
 
 ImageLoader *
 thumb_loader_get_image_loader (ThumbLoader *tl)
 {
-	ThumbLoaderPrivateData *priv;
-
 	g_return_val_if_fail (tl != NULL, NULL);
-
-	priv = tl->priv;
-	return priv->il;
+	return tl->priv->il;
 }
 
 
 char *
 thumb_loader_get_path (ThumbLoader *tl)
 {
-	ThumbLoaderPrivateData *priv;
-
 	g_return_val_if_fail (tl != NULL, NULL);
 
-	priv = tl->priv;
-	return priv->path;
+	if (tl->priv->uri == NULL)
+		return NULL;
+
+	return g_strdup (remove_scheme_from_uri (tl->priv->uri));
 }
 
 
@@ -474,20 +448,24 @@ thumb_loader_start (ThumbLoader *tl)
 
 	priv = tl->priv;
 
-	g_return_if_fail (priv->path != NULL);
+	g_return_if_fail (priv->uri != NULL);
 
 	if (priv->use_cache) {
 		time_t  mtime;
 
-		mtime = get_file_mtime (priv->path);
-		if (gnome_thumbnail_factory_has_valid_failed_thumbnail (priv->thumb_factory, priv->uri, mtime)) {
+		mtime = get_file_mtime (priv->uri);
+		if (gnome_thumbnail_factory_has_valid_failed_thumbnail (priv->thumb_factory, 
+									priv->e_uri, 
+									mtime)) {
 			g_signal_emit (G_OBJECT (tl),
 				       thumb_loader_signals[THUMB_ERROR],
 				       0);
 			return;
 		}
-		
-		cache_path = gnome_thumbnail_factory_lookup (priv->thumb_factory, priv->uri, mtime);
+
+		cache_path = gnome_thumbnail_factory_lookup (priv->thumb_factory, 
+							     priv->e_uri, 
+							     mtime);
 	}
 
 	if (cache_path != NULL) {
@@ -497,12 +475,12 @@ thumb_loader_start (ThumbLoader *tl)
 
 	} else {
 		priv->from_cache = FALSE;
-		image_loader_set_path (priv->il, priv->path);
+		image_loader_set_path (priv->il, remove_scheme_from_uri (priv->uri));
 
 		/* Check file dimensions. */
 
 		if ((priv->max_file_size != 0)
-		    && get_file_size (priv->path) > priv->max_file_size) {
+		    && get_file_size (priv->uri) > priv->max_file_size) {
 			if (priv->pixbuf != NULL) {
 				g_object_unref (priv->pixbuf);
 				priv->pixbuf = NULL;
@@ -584,9 +562,9 @@ thumb_loader_save_to_cache (ThumbLoader *tl)
 	if (priv->pixbuf == NULL) 
 		return FALSE;
 
-	mtime = get_file_mtime (priv->path);
-	
-	cache_path = gnome_thumbnail_path_for_uri (priv->uri, GNOME_THUMBNAIL_SIZE_NORMAL);
+	mtime = get_file_mtime (priv->uri);
+
+	cache_path = gnome_thumbnail_path_for_uri (priv->e_uri, GNOME_THUMBNAIL_SIZE_NORMAL);
 	cache_dir = remove_level_from_path (cache_path);
 	g_free (cache_path);
 
@@ -596,7 +574,7 @@ thumb_loader_save_to_cache (ThumbLoader *tl)
 	if (ensure_dir_exists (cache_dir, THUMBNAIL_DIR_PERMISSIONS)) 
 		gnome_thumbnail_factory_save_thumbnail (priv->thumb_factory,
 							priv->pixbuf,
-							priv->uri,
+							priv->e_uri,
 							mtime);
 
 	g_free (cache_dir);
@@ -624,8 +602,8 @@ thumb_loader_done_cb (ImageLoader *il,
 
 	if (pixbuf == NULL) {
 		gnome_thumbnail_factory_create_failed_thumbnail (priv->thumb_factory,
-								 priv->uri,
-								 get_file_mtime (priv->path));
+								 priv->e_uri,
+								 get_file_mtime (priv->uri));
 		g_signal_emit (G_OBJECT (tl), thumb_loader_signals[THUMB_ERROR], 0);
 		return;
 	}
@@ -700,7 +678,9 @@ thumb_loader_error_cb (ImageLoader *il,
 			priv->pixbuf = NULL;
 		}
 
-		gnome_thumbnail_factory_create_failed_thumbnail (priv->thumb_factory, priv->uri, get_file_mtime (priv->path));
+		gnome_thumbnail_factory_create_failed_thumbnail (priv->thumb_factory, 
+								 priv->e_uri, 
+								 get_file_mtime (priv->uri));
 		g_signal_emit (G_OBJECT (tl), thumb_loader_signals[THUMB_ERROR], 0);
 
 		return;
@@ -710,7 +690,7 @@ thumb_loader_error_cb (ImageLoader *il,
 	priv->from_cache = FALSE;
 	g_warning ("Thumbnail image in cache failed to load, trying to recreate.");
 	
-	image_loader_set_path (priv->il, priv->path);
+	image_loader_set_path (priv->il, remove_scheme_from_uri (priv->uri));
 	image_loader_start (priv->il);
 }
 
