@@ -33,17 +33,20 @@ typedef struct {
 	GtkWidget    * list_view;       /* The widget that shows the list. */
 	GtkWidget    * root_widget;     /* The widget that contains all. */
 	GtkCellRenderer  *text_renderer;
-	gboolean       use_underline;
+
+	gboolean     single_click;
+	gboolean     dirs_only;
+	GtkTreePath *hover_path;
 } CatalogList;
 
 
-CatalogList *  catalog_list_new                      (gboolean use_underline);
+CatalogList *  catalog_list_new                      (gboolean single_click_policy);
 
 void           catalog_list_free                     (CatalogList *cat_list);
 
 gboolean       catalog_list_refresh                  ();
 
-void           catalog_list_update_underline         (CatalogList *cat_list);
+void           catalog_list_update_click_policy      (CatalogList *cat_list);
 
 gchar *        catalog_list_get_path_from_tree_path  (CatalogList *cat_list,
 						      GtkTreePath *path);
@@ -81,5 +84,7 @@ void           catalog_list_change_to                (CatalogList *cat_list,
 void           catalog_list_select_iter              (CatalogList *cat_list,
 						      GtkTreeIter *iter);
 
+void           catalog_list_show_dirs_only           (CatalogList *cat_list,
+						      gboolean     value);
 
 #endif /* CATALOG_LIST_H */
