@@ -28,10 +28,12 @@
 #include <time.h>
 #include <locale.h>
 #include <ctype.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
+
 #include "catalog-web-exporter.h"
 #include "comments.h"
 #include "file-utils.h"
@@ -120,7 +122,7 @@ image_data_new (const char *filename)
 
 	idata = g_new (ImageData, 1);
 
-	cdata = comments_load_comment (filename);
+	cdata = comments_load_comment (filename, TRUE);
 	idata->comment = comments_get_comment_as_xml_string (cdata, "&nbsp;<br />", "&nbsp;<br />");
 	if (cdata != NULL)
 		comment_data_free (cdata);
@@ -352,8 +354,8 @@ catalog_web_exporter_get_type ()
 
 
 CatalogWebExporter *
-catalog_web_exporter_new (GThumbWindow *window,
-			  GList        *file_list)
+catalog_web_exporter_new (GthWindow *window,
+			  GList     *file_list)
 {
 	CatalogWebExporter *ce;
 	GList              *scan;
@@ -602,7 +604,7 @@ comp_func_size (gconstpointer a, gconstpointer b)
 static int
 comp_func_none (gconstpointer a, gconstpointer b)
 {
-	return 0;
+	return -1;
 }
 
 
@@ -921,8 +923,11 @@ get_image_filename (CatalogWebExporter *ce,
 					(location ? "/" : ""),
 					file_name_from_path (idata->dest_filename),
 					NULL);
-	else
-		filename = g_strdup (idata->src_filename);
+	else {
+		filename = gnome_vfs_get_uri_from_local_path (idata->src_filename);
+		if (filename == NULL)
+			filename = g_strdup (idata->src_filename);
+	}
 	
 	return filename;
 }
@@ -1420,7 +1425,7 @@ gth_parsed_doc_print (GList              *document,
 			write_markup_escape_line (line, fout);
 			g_free (line);
 
-			write_line (" ", fout);
+			write_line (" &nbsp; ", fout);
 
 			line = get_exif_tag (idata->src_filename, 
 					     EXIF_TAG_MODEL);
@@ -1505,7 +1510,7 @@ export__copy_to_destination__step2 (GnomeVFSResult  result,
 	debug (DEBUG_INFO, "result: %s", gnome_vfs_result_to_string (result));
 
 	if (result != GNOME_VFS_OK) 
-		_gtk_error_dialog_run (GTK_WINDOW (ce->window->app), 
+		_gtk_error_dialog_run (GTK_WINDOW (ce->window), 
 				       gnome_vfs_result_to_string (result));
 
 	dlg_folder_delete (ce->window,
@@ -1899,7 +1904,9 @@ load_next_file (CatalogWebExporter *ce)
 	ce->file_to_load = ce->file_to_load->next;
 	if (ce->file_to_load == NULL) {
 		/* sort list */
-		ce->file_list = g_list_sort (ce->file_list, get_sortfunc (ce));
+		if ((ce->sort_method != GTH_SORT_METHOD_NONE)
+		    && (ce->sort_method != GTH_SORT_METHOD_MANUAL))
+			ce->file_list = g_list_sort (ce->file_list, get_sortfunc (ce));
 		if (ce->sort_type == GTK_SORT_DESCENDING)
 			ce->file_list = g_list_reverse (ce->file_list);
 
@@ -2363,31 +2370,6 @@ parse_theme_files (CatalogWebExporter *ce)
 }
 
 
-static char *
-get_temp_dir_name (void)
-{
-	static int  count = 0;
-	char       *tmp_dir = NULL;
-	
-	do {
-		g_free (tmp_dir);
-		tmp_dir = g_strdup_printf ("%s%s.%d.%d",
-					   g_get_tmp_dir (),
-					   "/gthumb",
-					   getpid (),
-					   count++);
-
-	} while (path_is_dir (tmp_dir));
-
-	if (mkdir (tmp_dir, 0700) != 0) {
-		g_free (tmp_dir);
-		return NULL;
-	}
-
-	return tmp_dir;
-}
-
-
 void
 catalog_web_exporter_export (CatalogWebExporter *ce)
 {
@@ -2403,7 +2385,7 @@ catalog_web_exporter_export (CatalogWebExporter *ce)
 	ce->tmp_location = get_temp_dir_name ();
 
 	if (ce->tmp_location == NULL) {
-		_gtk_error_dialog_run (GTK_WINDOW (ce->window->app), _("Could not create a temporary folder"));
+		_gtk_error_dialog_run (GTK_WINDOW (ce->window), _("Could not create a temporary folder"));
 		g_signal_emit (G_OBJECT (ce), catalog_web_exporter_signals[WEB_EXPORTER_DONE], 0);
 		return;
 	}
