@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,8 +25,11 @@
 #include <sys/stat.h>
 #include <string.h>
 #include <unistd.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
+#include <libgnome/gnome-help.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <glade/glade.h>
 
@@ -38,9 +41,9 @@
 #include "jpegutils/jpeg-data.h"
 #endif /* HAVE_LIBEXIF */
 
-#include "file-data.h"
+#include "file-utils.h"
 #include "gconf-utils.h"
-#include "gthumb-window.h"
+#include "gth-window.h"
 #include "gtk-utils.h"
 #include "image-loader.h"
 #include "main.h"
@@ -49,6 +52,7 @@
 #include "pixbuf-utils.h"
 #include "gthumb-stock.h"
 #include "gth-exif-utils.h"
+#include "rotation-utils.h"
 
 
 #define ROTATE_GLADE_FILE "gthumb_tools.glade"
@@ -56,18 +60,8 @@
 
 #define PREVIEW_SIZE 200
 
-enum {
-	TRAN_ROTATE_0,
-	TRAN_ROTATE_90,
-	TRAN_ROTATE_180,
-	TRAN_ROTATE_270,
-	TRAN_NONE,
-	TRAN_MIRROR,
-	TRAN_FLIP
-};
-
 typedef struct {
-	GThumbWindow *window;
+	GthWindow    *window;
 	GladeXML     *gui;
 
 	GtkWidget    *dialog;
@@ -78,32 +72,43 @@ typedef struct {
 	GtkWidget    *j_preview_image;
 	GtkWidget    *j_from_exif_checkbutton;
 
-	int           rot_type;    /* 0, 90 ,180, 270 */
-	int           tran_type;   /* mirror, flip */
+	gboolean      from_exif_data;
 	GList        *file_list;
 	GList        *files_changed_list;
 	GList        *current_image;
 	ImageLoader  *loader;
 	GdkPixbuf    *original_preview;
+	RotationData *rot_data;
 } DialogData;
 
 
+static void
+dialog_data_free (DialogData *data)
+{
+	if (data->files_changed_list != NULL) {
+		all_windows_notify_files_changed (data->files_changed_list);
+		path_list_free (data->files_changed_list);
+		data->files_changed_list = NULL;
+	}
+
+	all_windows_add_monitor ();
+
+	path_list_free (data->file_list);
+	if (data->loader != NULL)
+		g_object_unref (data->loader);
+	if (data->gui != NULL)
+		g_object_unref (data->gui);
+	g_free (data->rot_data);
+	g_free (data);
+}
+
+
 /* called when the main dialog is closed. */
 static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	if (data->files_changed_list != NULL) {
-		all_windows_notify_files_changed (data->files_changed_list);
-		path_list_free (data->files_changed_list);
-	}
-
-	all_windows_add_monitor ();
-
-	file_data_list_free (data->file_list);
-	g_object_unref (data->loader);
-	g_object_unref (data->gui);
-	g_free (data);
+	dialog_data_free (data);
 }
 
 
@@ -145,55 +150,6 @@ _gdk_pixbuf_scale_keep_aspect_ratio (GdkPixbuf *pixbuf,
 }
 
 
-static void
-update_rotation_from_exif_data (DialogData *data,
-				GList      *current_image)
-{
-#ifdef HAVE_LIBEXIF
-	FileData           *fd = current_image->data;
-	GthExifOrientation  orientation = get_exif_tag_short (fd->path, EXIF_TAG_ORIENTATION);
-
-	switch (orientation) {
-	case GTH_EXIF_ORIENTATION_TOP_LEFT:
-		data->rot_type = TRAN_ROTATE_0;
-		data->tran_type = TRAN_NONE;
-		break;
-	case GTH_EXIF_ORIENTATION_TOP_RIGHT:
-		data->rot_type = TRAN_ROTATE_0;
-		data->tran_type = TRAN_MIRROR;
-		break;
-	case GTH_EXIF_ORIENTATION_BOTTOM_RIGHT:
-		data->rot_type = TRAN_ROTATE_180;
-		data->tran_type = TRAN_NONE;
-		break;
-	case GTH_EXIF_ORIENTATION_BOTTOM_LEFT:
-		data->rot_type = TRAN_ROTATE_180;
-		data->tran_type = TRAN_MIRROR;
-		break;
-	case GTH_EXIF_ORIENTATION_LEFT_TOP:
-		data->rot_type = TRAN_ROTATE_90;
-		data->tran_type = TRAN_MIRROR;
-		break;
-	case GTH_EXIF_ORIENTATION_RIGHT_TOP:
-		data->rot_type = TRAN_ROTATE_90;
-		data->tran_type = TRAN_NONE;
-		break;
-	case GTH_EXIF_ORIENTATION_RIGHT_BOTTOM:
-		data->rot_type = TRAN_ROTATE_90;
-		data->tran_type = TRAN_FLIP;
-		break;
-	case GTH_EXIF_ORIENTATION_LEFT_BOTTOM:
-		data->rot_type = TRAN_ROTATE_270;
-		data->tran_type = TRAN_NONE;
-		break;
-	default:
-		data->rot_type = TRAN_ROTATE_0;
-		data->tran_type = TRAN_NONE;
-		break;
-	}
-#endif /* HAVE_LIBEXIF */
-}
-
 
 static void
 update_from_exif_data (DialogData *data)
@@ -208,42 +164,41 @@ update_from_exif_data (DialogData *data)
 	if (! from_exif)
 		return;
 	
-	update_rotation_from_exif_data (data, data->current_image);
+	update_rotation_from_exif_data (data->current_image->data, data->rot_data);
 
 	src_pixbuf = data->original_preview;
-	
 	if (src_pixbuf == NULL) 
 		return;
 
-	switch (data->rot_type) {
-	case TRAN_ROTATE_90:
+	switch (data->rot_data->rot_type) {
+	case GTH_TRANSFORM_ROTATE_90:
 		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
 		break;
-	case TRAN_ROTATE_180:
+	case GTH_TRANSFORM_ROTATE_180:
 		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, TRUE);
 		break;
-	case TRAN_ROTATE_270:
+	case GTH_TRANSFORM_ROTATE_270:
 		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
 		break;
 	default:
 		dest_pixbuf = src_pixbuf;
 		g_object_ref (dest_pixbuf);
-			break;
+		break;
 	}
 
 	src_pixbuf = dest_pixbuf;
 
-	switch (data->tran_type) {
-	case TRAN_MIRROR:
+	switch (data->rot_data->tran_type) {
+	case GTH_TRANSFORM_MIRROR:
 		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
 		break;
-	case TRAN_FLIP:
+	case GTH_TRANSFORM_FLIP:
 		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
 		break;
 	default:
 		dest_pixbuf = src_pixbuf;
 		g_object_ref (dest_pixbuf);
-			break;
+		break;
 	}
 	
 	g_object_unref (src_pixbuf);
@@ -289,7 +244,7 @@ image_loader_error_cb (ImageLoader  *il,
 static void
 load_current_image (DialogData *data)
 {
-	FileData *fd;
+	char *path;
 
 	if (data->current_image == NULL) {
 		gtk_widget_destroy (data->dialog);
@@ -299,12 +254,12 @@ load_current_image (DialogData *data)
 	gtk_widget_set_sensitive (data->j_button_vbox, FALSE);
 	gtk_widget_set_sensitive (data->j_revert_button, FALSE);
 
-	fd = data->current_image->data;
-	image_loader_set_path (data->loader, fd->path);
+	path = data->current_image->data;
+	image_loader_set_path (data->loader, path);
 	image_loader_start (data->loader);
 
-	data->rot_type = TRAN_ROTATE_0;
-	data->tran_type = TRAN_NONE;
+	data->rot_data->rot_type = GTH_TRANSFORM_ROTATE_0;
+	data->rot_data->tran_type = GTH_TRANSFORM_NONE;
 }
 
 
@@ -326,468 +281,120 @@ load_next_image (DialogData *data)
 }
 
 
-#ifdef HAVE_LIBEXIF
-
-
-static gboolean
-swap_fields (ExifContent *content,
-	     ExifTag      tag1,
-	     ExifTag      tag2)
-{
-	ExifEntry     *entry1;
-	ExifEntry     *entry2;
-	unsigned char *data;
-	unsigned int   size;
-	
-	entry1 = exif_content_get_entry (content, tag1);
-	if (entry1 == NULL) 
-		return FALSE;
-	
-	entry2 = exif_content_get_entry (content, tag2);
-	if (entry2 == NULL)
-		return FALSE;
-
-	data = entry1->data;
-	size = entry1->size;
-	
-	entry1->data = entry2->data;
-	entry1->size = entry2->size;
-	
-	entry2->data = data;
-	entry2->size = size;
-
-	return TRUE;
-}
-
-
-static ExifShort
-get_next_value_rotation_90 (int value)
-{
-	static ExifShort new_value [8] = {8, 7, 6, 5, 2, 1, 4, 3};
-	return new_value[value - 1];
-}
-
-
-static ExifShort
-get_next_value_mirror (int value)
-{
-	static ExifShort new_value [8] = {2, 1, 4, 3, 8, 7, 6, 5};
-	return new_value[value - 1];
-}
-
-
-static ExifShort
-get_next_value_flip (int value)
-{
-	static ExifShort new_value [8] = {4, 3, 2, 1, 6, 5, 8, 7};
-	return new_value[value - 1];
-}
-
-
 static void
-swap_xy_exif_fields (const char *filename,
-		     DialogData *data)
+notify_file_changed (DialogData *data,
+		     const char *filename,
+		     gboolean    notify_soon)
 {
-	JPEGData     *jdata;
-	ExifData     *edata;
-	unsigned int  i;
-
-	jdata = jpeg_data_new_from_file (filename);
-	if (jdata == NULL)
-		return;
-
-	edata = jpeg_data_get_exif_data (jdata);
-	if (edata == NULL) {
-		jpeg_data_unref (jdata);
-		return;
-	}
-
-	for (i = 0; i < EXIF_IFD_COUNT; i++) {
-		ExifContent *content = edata->ifd[i];
-
-		if ((content == NULL) || (content->count == 0)) 
-			continue;
-		
-		swap_fields (content, 
-			     EXIF_TAG_RELATED_IMAGE_WIDTH,
-			     EXIF_TAG_RELATED_IMAGE_LENGTH);
-		swap_fields (content, 
-			     EXIF_TAG_IMAGE_WIDTH,
-			     EXIF_TAG_IMAGE_LENGTH);
-		swap_fields (content, 
-			     EXIF_TAG_PIXEL_X_DIMENSION,
-			     EXIF_TAG_PIXEL_Y_DIMENSION);
-		swap_fields (content, 
-			     EXIF_TAG_X_RESOLUTION,
-			     EXIF_TAG_Y_RESOLUTION);
-		swap_fields (content, 
-			     EXIF_TAG_FOCAL_PLANE_X_RESOLUTION,
-			     EXIF_TAG_FOCAL_PLANE_Y_RESOLUTION);
-	}
-
-	jpeg_data_save_file (jdata, filename);
-
-	exif_data_unref (edata);
-	jpeg_data_unref (jdata);
+	if (notify_soon) {
+		GList *list = g_list_prepend (NULL, (char*) filename);
+		all_windows_notify_files_changed (list);
+		g_list_free (list);
+	} else 
+		data->files_changed_list = g_list_prepend (data->files_changed_list, g_strdup (filename));
 }
 
 
-static void
-update_orientation_field (const char *filename,
-			  DialogData *data)
-{
-	JPEGData     *jdata;
-	ExifData     *edata;
-	unsigned int  i;
-	gboolean      orientation_changed = FALSE;
-
-	jdata = jpeg_data_new_from_file (filename);
-	if (jdata == NULL)
-		return;
-
-	edata = jpeg_data_get_exif_data (jdata);
-	if (edata == NULL) {
-		jpeg_data_unref (jdata);
-		return;
-	}
-
-	for (i = 0; i < EXIF_IFD_COUNT; i++) {
-		ExifContent *content = edata->ifd[i];
-		ExifEntry   *entry;
-
-		if ((content == NULL) || (content->count == 0)) 
-			continue;
-
-		entry = exif_content_get_entry (content, EXIF_TAG_ORIENTATION);
-		if (!orientation_changed && (entry != NULL)) {
-			ExifByteOrder byte_order;
-			ExifShort     value;
-
-			byte_order = exif_data_get_byte_order (edata);
-			value = exif_get_short (entry->data, byte_order);
-
-			switch (data->rot_type) {
-			case TRAN_ROTATE_90:
-				value = get_next_value_rotation_90 (value);
-				break;
-			case TRAN_ROTATE_180:
-				value = get_next_value_rotation_90 (value);
-				value = get_next_value_rotation_90 (value);
-				break;
-			case TRAN_ROTATE_270:
-				value = get_next_value_rotation_90 (value);
-				value = get_next_value_rotation_90 (value);
-				value = get_next_value_rotation_90 (value);
-				break;
-			default:
-				break;
-			}
-			
-			switch (data->tran_type) {
-			case TRAN_MIRROR:
-				value = get_next_value_mirror (value);
-				break;
-			case TRAN_FLIP:
-				value = get_next_value_flip (value);
-				break;
-			default:
-				break;
-			}
-
-			exif_set_short (entry->data, byte_order, value);
-
-			orientation_changed = TRUE;
-		}
-	}
-
-	jpeg_data_save_file (jdata, filename);
-
-	exif_data_unref (edata);
-	jpeg_data_unref (jdata);
-}
-
-
-#endif
-
-
-static void
-apply_tranformation_jpeg (DialogData *data,
-			  GList      *current_image)
-{
-	FileData   *fd = current_image->data;
-	int         rot_type = data->rot_type; 
-	int         tran_type = data->tran_type; 
-	char       *line;
-	char       *tmp1, *tmp2;
-	static int  count = 0;
-	GError     *err = NULL;
-	GtkWindow  *parent = GTK_WINDOW (data->dialog);
-#ifdef HAVE_LIBJPEG
-	JXFORM_CODE transf;
-#else
-	char       *command;
-#endif
-	char       *e1, *e2;
-
-	if ((rot_type == TRAN_ROTATE_0) && (tran_type == TRAN_NONE))
-		return;
-
-	if (rot_type == TRAN_ROTATE_0)
-		tmp1 = g_strdup (fd->path);
-	else {
-		tmp1 = g_strdup_printf ("%s/gthumb.%d.%d",
-					g_get_tmp_dir (), 
-					getpid (),
-					count++);
-
-#ifdef HAVE_LIBJPEG
-		switch (rot_type) {
-		case TRAN_ROTATE_90:
-			transf = JXFORM_ROT_90;
-			break;
-		case TRAN_ROTATE_180:
-			transf = JXFORM_ROT_180;
-			break;
-		case TRAN_ROTATE_270:
-			transf = JXFORM_ROT_270;
-			break;
-		default:
-			transf = JXFORM_NONE;
-			break;
-		}
-
-		if (jpegtran (fd->path, tmp1, transf, &err) != 0) {
-			g_free (tmp1);
-			if (err != NULL) 
-				_gtk_error_dialog_from_gerror_run (parent, &err);
-			return;
-		}
-
-#else
-		switch (rot_type) {
-		case TRAN_ROTATE_90:
-			command = "-rotate 90"; 
-			break;
-		case TRAN_ROTATE_180:
-			command = "-rotate 180"; 
-			break;
-		case TRAN_ROTATE_270:
-			command = "-rotate 270"; 
-			break;
-		default:
-			break;
-		}
-
-		e1 = shell_escape (tmp1);
-		e2 = shell_escape (fd->path);
-	
-		line = g_strdup_printf ("jpegtran -copy all %s -outfile %s %s",
-					command, e1, e2);
-		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
-
-		g_free (e1);
-		g_free (e2);
-		g_free (line);
-
-		if (err != NULL) {
-			g_free (tmp1);
-			_gtk_error_dialog_from_gerror_run (parent, &err);
-			return;
-		}
-#endif
-	}
-
-	if (tran_type == TRAN_NONE)
-		tmp2 = g_strdup (tmp1);
-	else {
-		tmp2 = g_strdup_printf ("%s/gthumb.%d.%d",
-					g_get_tmp_dir (), 
-					getpid (),
-					count++);
-
-#ifdef HAVE_LIBJPEG
-		switch (tran_type) {
-		case TRAN_MIRROR:
-			transf = JXFORM_FLIP_H;
-			break;
-		case TRAN_FLIP:
-			transf = JXFORM_FLIP_V;
-			break;
-		default:
-			transf = JXFORM_NONE;
-			break;
-		}
-
-		if (jpegtran (tmp1, tmp2, transf, &err) != 0) {
-			g_free (tmp1);
-			if (err != NULL) 
-				_gtk_error_dialog_from_gerror_run (parent, &err);
-			return;
-		}
-
-#else
-		switch (tran_type) {
-		case TRAN_MIRROR:
-			command = "-flip horizontal"; 
-			break;
-		case TRAN_FLIP:
-			command = "-flip vertical"; 
-			break;
-		default:
-			return;
-		}
-
-		e1 = shell_escape (tmp2);
-		e2 = shell_escape (tmp1);
-
-		line = g_strdup_printf ("jpegtran -copy all %s -outfile %s %s",
-					command, e1, e2);
-		g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
-
-		g_free (e1);
-		g_free (e2);
-		g_free (line);
-
-		if (err != NULL) {
-			g_free (tmp1);
-			g_free (tmp2);
-			_gtk_error_dialog_from_gerror_run (parent, &err);
-			return;
-		}
-#endif
-	}
-
-	e1 = shell_escape (tmp2);
-	e2 = shell_escape (fd->path);
-
-	line = g_strdup_printf ("mv -f %s %s", e1, e2);
-	g_spawn_command_line_sync (line, NULL, NULL, NULL, &err);  
-
-	if (err != NULL) {
-		_gtk_error_dialog_from_gerror_run (parent, &err);
-
-	} else {
-#ifdef HAVE_LIBEXIF
-		if ((rot_type == TRAN_ROTATE_90) || (rot_type == TRAN_ROTATE_270))
-			swap_xy_exif_fields (fd->path, data);
-		/* update_orientation_field (fd->path, data); see bug #318828 */
-#endif
-
-		data->files_changed_list = g_list_prepend (data->files_changed_list, 
-							   g_strdup (fd->path));
-	}
-
-	g_free (e1);
-	g_free (e2);
-	g_free (line);
-	g_free (tmp1);
-	g_free (tmp2);
-}
-
-
-static void
-apply_transformation_generic (DialogData *data,
-			      GList      *current_image)
-{
-	FileData   *fd = current_image->data;
-	int         rot_type = data->rot_type; 
-	int         tran_type = data->tran_type; 
-	GdkPixbuf  *pixbuf1, *pixbuf2;
-	const char *mime_type;
-
-	if ((rot_type == TRAN_ROTATE_0) && (tran_type == TRAN_NONE))
-		return;
-
-	pixbuf1 = gdk_pixbuf_new_from_file (fd->path, NULL);
-	if (pixbuf1 == NULL)
-		return;
-
-	switch (rot_type) {
-	case TRAN_ROTATE_90:
-		pixbuf2 = _gdk_pixbuf_copy_rotate_90 (pixbuf1, FALSE);
-		break;
-	case TRAN_ROTATE_180:
-		pixbuf2 = _gdk_pixbuf_copy_mirror (pixbuf1, TRUE, TRUE);
-		break;
-	case TRAN_ROTATE_270:
-		pixbuf2 = _gdk_pixbuf_copy_rotate_90 (pixbuf1, TRUE);
-		break;
-	default:
-		pixbuf2 = pixbuf1;
-		g_object_ref (pixbuf2);
-		break;
-	}
-	g_object_unref (pixbuf1);
-
-	switch (tran_type) {
-	case TRAN_MIRROR:
-		pixbuf1 = _gdk_pixbuf_copy_mirror (pixbuf2, TRUE, FALSE);
-		break;
-	case TRAN_FLIP:
-		pixbuf1 = _gdk_pixbuf_copy_mirror (pixbuf2, FALSE, TRUE);
-		break;
-	default:
-		pixbuf1 = pixbuf2;
-		g_object_ref (pixbuf1);
-		break;
-	}
-	g_object_unref (pixbuf2);
-
-	mime_type = gnome_vfs_mime_type_from_name (fd->path);
-	if ((mime_type != NULL) && is_mime_type_writable (mime_type)) {
-		GError      *error = NULL;
-		const char  *image_type = mime_type + 6;
-		if (! _gdk_pixbuf_save (pixbuf1, 
-					fd->path, 
-					image_type, 
-					&error, 
-					NULL))
-			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->window->app), &error);
-	}
-	g_object_unref (pixbuf1);
-
-	data->files_changed_list = g_list_prepend (data->files_changed_list, 
-						   g_strdup (fd->path));
-}
-
 
 static void
 apply_transformation (DialogData *data,
-		      GList      *current_image)
+		      GList      *current_image,
+		      gboolean    notify_soon)
 {
-	FileData    *fd = current_image->data;
-	char        *dir;
-	struct stat  buf;
-
+	char             *path = current_image->data;
+	char             *dir;
+	GnomeVFSFileInfo  info;
+	GtkWindow  	 *window = GTK_WINDOW (data->dialog);
+        
 	/* Check directory permissions. */
 
-	dir = remove_level_from_path (fd->path);
+	dir = remove_level_from_path (path);
 	if (! check_permissions (dir, R_OK | W_OK | X_OK)) {
-		char *utf8_path;
-		utf8_path = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+		char *utf8_path = g_filename_display_name (dir);
 		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
 				       _("You don't have the right permissions to create images in the folder \"%s\""),
 				       utf8_path);
 		g_free (utf8_path);
 		g_free (dir);
-
 		return;
 	} 
 	g_free (dir);
 
 	/**/
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton)))
-		update_rotation_from_exif_data (data, current_image);
-
-	stat (fd->path, &buf);
-
-	if (image_is_jpeg (fd->path)) 
-		apply_tranformation_jpeg (data, current_image);
+	if (data->from_exif_data)
+		update_rotation_from_exif_data (path, data->rot_data);
+	gnome_vfs_get_file_info (path, &info, GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (image_is_jpeg (path)) 
+		apply_transformation_jpeg (window, path, data->rot_data);
 	else 
-		apply_transformation_generic (data, current_image);
+		apply_transformation_generic (window, path, data->rot_data);
+	gnome_vfs_set_file_info (path, &info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS|GNOME_VFS_SET_FILE_INFO_OWNER);
 
-	chmod (fd->path, buf.st_mode);
+	notify_file_changed (data, path, notify_soon);
+}
+
+
+static void
+apply_transformation_to_all (DialogData *data)
+{
+	GladeXML  *gui;
+	GtkWidget *dialog;
+	GtkWidget *label;
+	GtkWidget *bar;
+	GList     *scan;
+	int        i, n;
+
+	gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE, 
+			     NULL,
+			     NULL);
+
+	dialog = glade_xml_get_widget (gui, "progress_dialog");
+	label = glade_xml_get_widget (gui, "progress_info");
+	bar = glade_xml_get_widget (gui, "progress_progressbar");
+
+	n = g_list_length (data->current_image);
+
+	if (data->dialog == NULL)
+		gtk_window_set_transient_for (GTK_WINDOW (dialog),
+					      GTK_WINDOW (data->window));
+	else {
+		gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE); 
+		gtk_window_set_transient_for (GTK_WINDOW (dialog),
+					      GTK_WINDOW (data->dialog));
+	}
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE); 
+	gtk_widget_show (dialog);
+
+	while (gtk_events_pending())
+		gtk_main_iteration();
+
+	i = 0;
+	for (scan = data->current_image; scan; scan = scan->next) {
+		char *path = scan->data;
+		char *name;
+
+		name = g_filename_display_basename (path);
+		_gtk_label_set_filename_text (GTK_LABEL (label), name);
+		g_free (name);
+
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (bar),
+					       (gdouble) (i + 0.5) / n);
+		
+		while (gtk_events_pending())
+			gtk_main_iteration();
+		
+		apply_transformation (data, scan, FALSE);
+		i++;
+	}
+	
+	gtk_widget_destroy (dialog);
+	g_object_unref (gui);
+
+	if (data->dialog == NULL)
+		dialog_data_free (data);
+	else
+		gtk_widget_destroy (data->dialog);
 }
 
 
@@ -798,55 +405,13 @@ ok_clicked (GtkWidget  *button,
 	gboolean to_all;
 
 	to_all = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_apply_to_all_checkbutton));
+	data->from_exif_data = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton));
 
 	if (to_all) {
-		GladeXML  *gui;
-		GtkWidget *dialog;
-		GtkWidget *label;
-		GtkWidget *bar;
-		GList     *scan;
-		int        i, n;
-
 		gtk_widget_hide (data->dialog);
-
-		gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE, 
-				     NULL,
-				     NULL);
-
-		dialog = glade_xml_get_widget (gui, "progress_dialog");
-		label = glade_xml_get_widget (gui, "progress_info");
-		bar = glade_xml_get_widget (gui, "progress_progressbar");
-
-		n = g_list_length (data->current_image);
-
-		gtk_widget_show (dialog);
-
-		while (gtk_events_pending())
-			gtk_main_iteration();
-
-		i = 0;
-		for (scan = data->current_image; scan; scan = scan->next) {
-			FileData *fd = scan->data;
-
-			_gtk_label_set_filename_text (GTK_LABEL (label), fd->name);
-			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (bar),
-						       (gdouble) (i + 1) / n);
-
-			while (gtk_events_pending())
-				gtk_main_iteration();
-
-			apply_transformation (data, scan);
-
-			i++;
-		}
-
-		gtk_widget_destroy (dialog);
-		g_object_unref (gui);
-
-		gtk_widget_destroy (data->dialog);
-
+		apply_transformation_to_all (data);
 	} else {
-		apply_transformation (data, data->current_image);
+		apply_transformation (data, data->current_image, TRUE);
 		load_next_image (data);
 	}
 }
@@ -855,10 +420,10 @@ ok_clicked (GtkWidget  *button,
 static int 
 get_next_rot (int rot)
 {
-	int next_rot [4] = {TRAN_ROTATE_90, 
-			    TRAN_ROTATE_180, 
-			    TRAN_ROTATE_270,
-			    TRAN_ROTATE_0};
+	int next_rot [4] = {GTH_TRANSFORM_ROTATE_90, 
+			    GTH_TRANSFORM_ROTATE_180, 
+			    GTH_TRANSFORM_ROTATE_270,
+			    GTH_TRANSFORM_ROTATE_0};
 	return next_rot [rot];
 }
 
@@ -867,8 +432,8 @@ static void
 revert_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	data->rot_type = TRAN_ROTATE_0;
-	data->tran_type = TRAN_NONE;
+	data->rot_data->rot_type = GTH_TRANSFORM_ROTATE_0;
+	data->rot_data->tran_type = GTH_TRANSFORM_NONE;
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton), FALSE);
 
@@ -884,12 +449,12 @@ rot90_clicked (GtkWidget  *button,
 	GdkPixbuf *src_pixbuf;
 	GdkPixbuf *dest_pixbuf;
 
-	if (data->tran_type == TRAN_NONE)
-		data->rot_type = get_next_rot (data->rot_type);
+	if (data->rot_data->tran_type == GTH_TRANSFORM_NONE)
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
 	else {
-		data->rot_type = get_next_rot (data->rot_type);
-		data->rot_type = get_next_rot (data->rot_type);
-		data->rot_type = get_next_rot (data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
 	}
 
 	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
@@ -912,12 +477,12 @@ rot270_clicked (GtkWidget  *button,
 	GdkPixbuf *src_pixbuf;
 	GdkPixbuf *dest_pixbuf;
 
-	if (data->tran_type == TRAN_NONE) {
-		data->rot_type = get_next_rot (data->rot_type);
-		data->rot_type = get_next_rot (data->rot_type);
-		data->rot_type = get_next_rot (data->rot_type);
+	if (data->rot_data->tran_type == GTH_TRANSFORM_NONE) {
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
 	} else
-		data->rot_type = get_next_rot (data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
 
 	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
 
@@ -944,16 +509,16 @@ mirror_clicked (GtkWidget  *button,
 	if (src_pixbuf == NULL) 
 		return;
 
-	if (data->tran_type == TRAN_FLIP) {
-		data->tran_type = TRAN_NONE;
-		data->rot_type = get_next_rot (data->rot_type);
-		data->rot_type = get_next_rot (data->rot_type);
+	if (data->rot_data->tran_type == GTH_TRANSFORM_FLIP) {
+		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
 
-	} else if (data->tran_type == TRAN_MIRROR)
-		data->tran_type = TRAN_NONE;
+	} else if (data->rot_data->tran_type == GTH_TRANSFORM_MIRROR)
+		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
 
 	else
-		data->tran_type = TRAN_MIRROR;
+		data->rot_data->tran_type = GTH_TRANSFORM_MIRROR;
 	
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
 	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
@@ -970,16 +535,16 @@ flip_clicked (GtkWidget  *button,
 	GdkPixbuf *src_pixbuf;
 	GdkPixbuf *dest_pixbuf;
 
-	if (data->tran_type == TRAN_MIRROR) {
-		data->tran_type = TRAN_NONE;
-		data->rot_type = get_next_rot (data->rot_type);
-		data->rot_type = get_next_rot (data->rot_type);
+	if (data->rot_data->tran_type == GTH_TRANSFORM_MIRROR) {
+		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
+		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
 
-	} else if (data->tran_type == TRAN_FLIP)
-		data->tran_type = TRAN_NONE;
+	} else if (data->rot_data->tran_type == GTH_TRANSFORM_FLIP)
+		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
 
 	else
-		data->tran_type = TRAN_FLIP;
+		data->rot_data->tran_type = GTH_TRANSFORM_FLIP;
 
 	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
 
@@ -1036,7 +601,7 @@ from_exif_toggled_cb (GtkToggleButton *button,
 
 
 void
-dlg_jpegtran (GThumbWindow *window)
+dlg_jpegtran (GthWindow *window)
 {
 	DialogData  *data;
 	GtkWidget   *j_image_vbox;
@@ -1051,7 +616,8 @@ dlg_jpegtran (GThumbWindow *window)
 	GtkWidget   *reset_image;
 	GList       *list;
 
-	list = gth_file_list_get_selection_as_fd (window->file_list);
+
+	list = gth_window_get_file_list_selection (window);
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		return;
@@ -1061,6 +627,7 @@ dlg_jpegtran (GThumbWindow *window)
 
 	data = g_new0 (DialogData, 1);
 
+	data->window = window;
 	data->file_list = list;
 	data->current_image = list;
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" ROTATE_GLADE_FILE, NULL,
@@ -1069,11 +636,13 @@ dlg_jpegtran (GThumbWindow *window)
 	if (! data->gui) {
 		g_warning ("Could not find " ROTATE_GLADE_FILE "\n");
 		if (data->file_list != NULL) 
-			g_list_free (data->file_list);
+			path_list_free (data->file_list);
 		g_free (data);
 		return;
 	}
 
+	data->rot_data = rotation_data_new ();
+
 	/* Get the widgets. */
 
 	data->dialog = glade_xml_get_widget (data->gui, "jpeg_rotate_dialog");
@@ -1172,9 +741,64 @@ dlg_jpegtran (GThumbWindow *window)
 
 	all_windows_remove_monitor ();
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window->app));
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE); 
 	gtk_widget_show_all (data->dialog);
 
 	load_current_image (data);
 }
+
+
+void
+dlg_apply_jpegtran (GthWindow    *window,
+		    GthTransform  rot_type,
+		    GthTransform  tran_type)
+{
+	DialogData  *data;
+	GList       *list;
+
+	list = gth_window_get_file_list_selection (window);
+	if (list == NULL) {
+		g_warning ("No file selected.");
+		return;
+	}
+
+	all_windows_remove_monitor ();
+
+	data = g_new0 (DialogData, 1);
+	data->window = window;
+	data->file_list = list;
+	data->current_image = list;
+
+	data->rot_data = rotation_data_new();
+	data->rot_data->rot_type = rot_type;
+	data->rot_data->tran_type = tran_type;
+
+	apply_transformation_to_all (data);
+}
+
+
+
+void
+dlg_apply_jpegtran_from_exif (GthWindow *window)
+{
+	DialogData  *data;
+	GList       *list;
+
+	list = gth_window_get_file_list_selection (window);
+	if (list == NULL) {
+		g_warning ("No file selected.");
+		return;
+	}
+
+	all_windows_remove_monitor ();
+
+	data = g_new0 (DialogData, 1);
+	data->window = window;
+	data->file_list = list;
+	data->current_image = list;
+	data->from_exif_data = TRUE;
+	data->rot_data = rotation_data_new();
+
+	apply_transformation_to_all (data);
+}
